<?php

namespace App\Http\Controllers;

use App\Helpers\General;
use App\Http\Requests\CompanyRequest;
use App\Models\Company;
use Illuminate\Http\Request;
use Yajra\DataTables\DataTables;

class CompanyController extends Controller
{
    private string $redirectRoute = 'company.index';

    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        return view('company.index');
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     */
    public function datatable()
    {
        $query = Company::query();
        return DataTables::of($query)
            ->editColumn('status', function($company) {
                return view('common.status', [
                    'model' => $company,
                    'url' => route('company.status', $company)
                ])->render();
            })
            ->addColumn('actions', function ($company) {
                return actions(route('company.edit', $company), route('company.destroy', $company), $company->id);
            })
            ->setRowId(function ($company) {
                return 'delete-row-' . $company->id;
            })
            ->escapeColumns([])
            ->make(true);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('company.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(CompanyRequest $request)
    {
        try {
            $request->merge([
                'status' => $request->status ?? 0
            ]);

            Company::create([
                'name' => $request->name,
                'address' => $request->address,
                'registration_no' => $request->registration_no,
                'status' => $request->status,
            ]);

            return redirect()->route($this->redirectRoute);
        } catch (\Exception $e) {
            return redirect()->back()->with('error', $e->getMessage());
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Company $company)
    {
        return view('company.edit', compact('company'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Company $company)
    {
        try {
            $request->merge([
                'status' => $request->status ?? 0,
            ]);

            $update = $company->update($request->all());
            if ($update) {
                return redirect()->route($this->redirectRoute)->with('success', 'Record updated successfully.');
            }

            return redirect()->withInput()->back()->with('error', 'Failed to save the record, please try again.');

        } catch (\Exception $e) {
            return redirect()->back()->withInput()->with('error',  General::getMessage($e));
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Company $company)
    {
        try {
            $company->delete();
            return response()->json([
                'status' => true,
                'message' => 'Record deleted successfully.'
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => General::getMessage($e)
            ]);
        }
    }

    public function status(Request $request, Company $company) {
        $company->update([
            'status' => $request->status ?? 0
        ]);
        return response()->json([
            'status' => true,
            'data' => view('common.status', [
                'model' => $company,
                'url' => route('company.status', $company)
            ])->render(),
            'message' => 'Status updated successfully.',
        ]);
    }

}
