<?php

namespace App\Http\Controllers;

use App\Helpers\General;
use App\Http\Requests\TcpRequest;
use App\Models\Tcp;
use Illuminate\Http\Request;
use Yajra\DataTables\DataTables;

class TcpController extends Controller
{
    private string $redirectRoute = 'tcp.index';

    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        return view('tcp.index');
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     */
    public function datatable()
    {
        $query = Tcp::query();
        return DataTables::of($query)
            ->addColumn('image', function ($tcp) {
                return imagePreview($tcp->image);
            })
            ->editColumn('status', function($tcp) {
                return view('common.status', [
                    'model' => $tcp,
                    'url' => route('tcp.status', $tcp)
                ])->render();
            })
            ->addColumn('actions', function ($tcp) {
                return actions(route('tcp.edit', $tcp), route('tcp.destroy', $tcp), $tcp->id);
            })
            ->setRowId(function ($tcp) {
                return 'delete-row-' . $tcp->id;
            })
            ->escapeColumns([])
            ->make(true);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('tcp.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(TcpRequest $request)
    {
        try {
            $request->merge([
                'status' => $request->status ?? 0
            ]);
            if ($request->hasFile('file')) {
                $request->merge([
                    'image' => General::uploadFile($request, 'file', 'tcp'),
                ]);
            }
            Tcp::create([
                'name' => $request->name,
                'town' => $request->town,
                'zone' => $request->zone,
                'uc' => $request->uc,
                'lat' => $request->lat,
                'long' => $request->long,
                'image' => $request->image,
                'status' => $request->status,
            ]);

            return redirect()->route($this->redirectRoute);
        } catch (\Exception $e) {
            return redirect()->back()->with('error', $e->getMessage());
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Tcp $tcp)
    {
        return view('tcp.edit', compact('tcp'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Tcp $tcp)
    {
        try {
            $request->merge([
                'status' => $request->status ?? 0,
            ]);
            if ($request->hasFile('file')) {
                $request->merge([
                    'image' => General::uploadFile($request, 'file', 'tcp'),
                ]);
            }

            $update = $tcp->update($request->all());
            if ($update) {
                return redirect()->route($this->redirectRoute)->with('success', 'Record updated successfully.');
            }

            return redirect()->withInput()->back()->with('error', 'Failed to save the record, please try again.');

        } catch (\Exception $e) {
            return redirect()->back()->withInput()->with('error',  General::getMessage($e));
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Tcp $tcp)
    {
        try {
            $tcp->delete();
            return response()->json([
                'status' => true,
                'message' => 'Record deleted successfully.'
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => General::getMessage($e)
            ]);
        }
    }

    public function status(Request $request, Tcp $tcp) {
        $tcp->update([
            'status' => $request->status ?? 0
        ]);
        return response()->json([
            'status' => true,
            'data' => view('common.status', [
                'model' => $tcp,
                'url' => route('tcp.status', $tcp)
            ])->render(),
            'message' => 'Status updated successfully.',
        ]);
    }
}
