<?php

namespace App\Http\Controllers;

use App\Helpers\General;
use App\Http\Requests\TcpUserRequest;
use App\Http\Requests\TcpUserUpdateRequest;
use App\Models\Tcp;
use App\Models\TcpUser;
use App\Models\User;
use Illuminate\Http\Request;
use Yajra\DataTables\DataTables;

class TcpUserController extends Controller
{
    private string $redirectRoute = 'user.index';
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        return view('tcp_user.index');
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     */
    public function datatable()
    {
        $query = TcpUser::query();
        return DataTables::of($query)
            ->editColumn('status', function($user) {
                return view('common.status', [
                    'model' => $user,
                    'url' => route('user.status', $user)
                ])->render();
            })
            ->addColumn('tcp', function ($user) {
                return '<span class="badge badge-light-success">'.$user?->tcp?->name ?? 'N/A'.'</span>';
            })
            ->addColumn('email', function ($user) {
                return '<span class="badge badge-light-info">'.$user?->user?->email ?? 'N/A'.'</span>';
            })
            ->addColumn('actions', function ($user) {
                return actions(route('user.edit', $user), route('user.destroy', $user), $user->id);
            })
            ->setRowId(function ($user) {
                return 'delete-row-' . $user->id;
            })
            ->escapeColumns([])
            ->make(true);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $tcps = Tcp::isActive()->get(['id', 'name']);
        return view('tcp_user.create', compact('tcps'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(TcpUserRequest $request)
    {
        try {
            $request->merge([
                'status' => $request->status ?? 0
            ]);

            $user = User::create([
                'name' =>  $request->name ?? '',
                'role' => 'User',
                'email' => $request->email ?? '',
                'password' => bcrypt($request->password ?? ''),
            ]);

            TcpUser::create([
                'user_id' => $user->id ?? '',
                'name' => $request->name ?? '',
                'tcp_id' => $request->tcp_id ?? '',
                'shift' => $request->shift ?? '',
                'cnic' => $request->cnic ?? '',
                'phone' => $request->phone ?? '',
                'status' => $request->status ?? '0',
            ]);

            return redirect()->route($this->redirectRoute);
        } catch (\Exception $e) {
            return redirect()->back()->withInput()->with('error', $e->getMessage());
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(TcpUser $user)
    {
        $tcps = Tcp::isActive()->get(['id', 'name']);
        return view('tcp_user.edit', compact('user', 'tcps'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(TcpUserUpdateRequest $request, TcpUser $user)
    {
        try {
            $request->merge([
                'status' => $request->status ?? 0,
            ]);

            $userData = [
                'name' =>  $request->name ?? '',
                'role' => 'User',
                'email' => $request->email ?? '',
            ];

            if (isset($request->password)) {
                $userData['password'] = bcrypt($request->password ?? '');
            }
            User::where('id', $request->user_id)->update($userData);

            $update = $user->update([
                'name' => $request->name ?? '',
                'tcp_id' => $request->tcp_id ?? '',
                'shift' => $request->shift ?? '',
                'cnic' => $request->cnic ?? '',
                'phone' => $request->phone ?? '',
                'status' => $request->status ?? '0',
            ]);
            if ($update) {
                return redirect()->route($this->redirectRoute)->with('success', 'Record updated successfully.');
            }

            return redirect()->back()->withInput()->with('error', 'Failed to save the record, please try again.');

        } catch (\Exception $e) {
            return redirect()->back()->withInput()->with('error',  General::getMessage($e));
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(TcpUser $user)
    {
        try {
            $user->delete();
            return response()->json([
                'status' => true,
                'message' => 'Record deleted successfully.'
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => General::getMessage($e)
            ]);
        }
    }

    public function status(Request $request, TcpUser $user) {

        $user->update([
            'status' => $request->status ?? 0
        ]);
        return response()->json([
            'status' => true,
            'data' => view('common.status', [
                'model' => $user,
                'url' => route('user.status', $user)
            ])->render(),
            'message' => 'Status updated successfully.',
        ]);
    }
}
