<?php

namespace App\Http\Controllers;

use App\Helpers\General;
use App\Http\Requests\VehicleRequest;
use App\Models\Company;
use App\Models\Vehicle;
use Illuminate\Http\Request;
use Yajra\DataTables\DataTables;

class VehicleController extends Controller
{
    private string $redirectRoute = 'vehicle.index';

    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        return view('vehicle.index');
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     */
    public function datatable()
    {
        $query = Vehicle::query();
        return DataTables::of($query)
            ->editColumn('status', function($vehicle) {
                return view('common.status', [
                    'model' => $vehicle,
                    'url' => route('vehicle.status', $vehicle)
                ])->render();
            })
            ->addColumn('company', function ($vehicle) {
                return '<span class="badge badge-light-success">'.$vehicle?->company?->name ?? 'N/A'.'</span>';
            })
            ->addColumn('actions', function ($vehicle) {
                return actions(route('vehicle.edit', $vehicle), route('vehicle.destroy', $vehicle), $vehicle->id);
            })
            ->setRowId(function ($company) {
                return 'delete-row-' . $company->id;
            })
            ->escapeColumns([])
            ->make(true);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $companies = Company::isActive()->get(['id', 'name']);
        return view('vehicle.create', compact('companies'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(VehicleRequest $request)
    {
        try {
            $request->merge([
                'status' => $request->status ?? 0
            ]);

            Vehicle::create([
                'company_id' => $request->company_id,
                'registration_no' => $request->registration_no,
                'category' => $request->category,
                'town' => $request->town,
                'zone' => $request->zone,
                'uc' => $request->uc,
                'name' => $request->name,
                'status' => $request->status,
            ]);

            return redirect()->route($this->redirectRoute);
        } catch (\Exception $e) {
            return redirect()->back()->withInput()->with('error', $e->getMessage());
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Vehicle $vehicle)
    {
        $companies = Company::isActive()->get(['id', 'name']);
        return view('vehicle.edit', compact('vehicle', 'companies'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(VehicleRequest $request, Vehicle $vehicle)
    {
        try {
            $request->merge([
                'status' => $request->status ?? 0,
            ]);
            $update = $vehicle->update($request->all());
            if ($update) {
                return redirect()->route($this->redirectRoute)->with('success', 'Record updated successfully.');
            }

            return redirect()->withInput()->back()->with('error', 'Failed to save the record, please try again.');

        } catch (\Exception $e) {
            return redirect()->back()->withInput()->with('error',  General::getMessage($e));
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Vehicle $vehicle)
    {
        try {
            $vehicle->delete();
            return response()->json([
                'status' => true,
                'message' => 'Record deleted successfully.'
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => General::getMessage($e)
            ]);
        }
    }

    public function status(Request $request, Vehicle $vehicle) {
        $vehicle->update([
            'status' => $request->status ?? 0
        ]);
        return response()->json([
            'status' => true,
            'data' => view('common.status', [
                'model' => $vehicle,
                'url' => route('vehicle.status', $vehicle)
            ])->render(),
            'message' => 'Status updated successfully.',
        ]);
    }
}
